;**********************************************************
;*                  LEDUHR.ASM V1.0                       *
;* Steuerprogramm fuer Uhr mit 60 LEDs                    *
;* Ansteuerung der LEDS ueber den LED-Driver MAX7219      *
;* Assemblerprogramm fuer den Mikrocontroller AT89C2051,  *
;* Quarzfrequenz 12 MHz                                   *
;**********************************************************
;* World Of Electronics                                   *
;* http://www.woe.de.vu                                   *
;* (c) WOE, 1999                                          *
;**********************************************************

DIN equ p1.4        ; DIN an Port 1.4
CLK equ p1.6        ; CLK an Port 1.6
LOAD equ p1.5       ; LOAD an Port 1.5

MODESELECT equ p3.3  ; LED-Modus auswaehlen (Jumper)


rev equ 0            ; Reverse-Bit (LED-Kette zu- oder abnehmend)
clockmode equ 1      ; LED-Modus


org 0h
	jmp anfang

ret_command:
	reti

org 0Bh                      ; Timer 0-Interrupt
	call ret_command     ; Interrupt wieder freigeben
	djnz r7,ret_command
	mov r7,#200
	djnz r6,ret_command
	mov r6,#20           ; 250s*200*20 = 1s
	pop acc              ; Stack bereinigen
	pop acc
	jmp prg              ; LEDs im Sekundentakt updaten


anfang:
	mov c,MODESELECT ; eingestellten Mode einlesen
	mov clockmode,c
	clr c

	call ledout_ini  ; MAX7219 initialisieren
	mov dptr,#daten
	mov r7,#200
	mov r6,#20
	mov r5,#0
	mov r2,#1
	clr rev

	mov TMOD,#2      ; Timer 0: Reload-Mode
	mov TH0,#6       ; Zeitbasis 250 s
	mov TL0,#6
	setb ET0         ; Timer 0 Interruptfreigabe
	setb EA          ; globale Interruptfreigabe
	setb TR0         ; Timer 0 starten
	jmp wait_int

prg:
	mov a,r2
	dec a
	rl a
	rl a
	rl a             ; 3x rl = Multiplikation mit 8
	cjne a,5,comp    ; Akku mit r5 (=Adr.5) vergleichen
	jmp kl
comp:
	jc kl

gr:                      ; Akku > r5
	jb rev,rev1
	mov r3,#0
	jmp next
rev1:
	mov r3,#0ffh
	jmp next

kl:                      ; Akku <= r5
	mov a,r5
	mov b,#8
	div ab
	inc a
	cjne a,2,l1      ; Akku mit r2 (=Adr.2) vergleichen
	mov a,b
	movc a,@a+dptr   ; Wert aus Tabelle holen
	mov r3,a

next:
	call ledout      ; Daten an MAX7219 senden
	inc r2
	cjne r2,#9,prg

	mov r2,#1
	inc r5
	cjne r5,#60,wait_int
	mov r5,#0
	jnb clockmode,wait_int ; bei clockmode=1 Reversebit toggeln
	cpl rev
	jb rev,revdat
	mov dptr,#daten
	jmp wait_int
revdat:
	mov dptr,#daten_rev

wait_int:                ; warten auf Interrupt
	jmp wait_int

l1:
	jb rev,rev2
	mov r3,#0ffh
	jmp next
rev2:
	mov r3,#0
	jmp next


;*********************************************************
; Initialisierung:
; (verwendet R0, R2, R3, Akku)

ledout_ini:
	clr DIN
	clr CLK
	clr LOAD
       
; Decode-Mode: no decode
	mov r2,#09h     ; Adresse des Decode Mode Registers
	mov r3,#0       ; keine Decodierung
	call ledout

; Intensity auf Maximum stellen:
	mov r2,#0Ah
	mov r3,#0Fh
	call ledout

; Scan Limit auf 8 aktive Displays stellen:
	mov r2,#0Bh
	mov r3,#07h
	call ledout

; Display test mode = off:
	mov r2,#0Fh     ; Adresse des Display Test Registers
	mov r3,#0h      ; normal operation
	call ledout

; Display auf 0 setzen:
	mov r0,#8       ; 8 Digits
led_ini_10:
	mov a,r0
	mov r2,a        ; jedes Digit
	mov r3,#0       ; auf 0
	call ledout
	djnz r0,led_ini_10

; Shutdown auf off = Display einschalten:
	mov r2,#0Ch
	mov r3,#01h     ; normal operation
	call ledout

	ret


;************************************************
; Datenausgabe an MAX7219
; Parameter:
; R2 = Adressbyte fuer MAX7219
; R3 = Datenbyte fuer MAX7219

ledout: push acc
	mov a,r0
	push acc
	mov a,r1
	push acc
	mov acc,r2

	mov r1,#2       ;2-mal 8 Bits
lo_5:   mov r0,#8       ;je 8 Bits
lo_10:  jb acc.7,lo_20
	clr DIN         ;0 uebertragen
	jmp lo_30
lo_20:  setb DIN        ;1 uebertragen
lo_30:  setb CLK        ;Takt ausgeben
	clr CLK
	rl a            ;naechstes Bit
	djnz r0,lo_10   ;(8 mal)
	mov a,r3        ;Datenbyte in Akku
	djnz r1,lo_5    ;nochmal, aber jetzt mit Datenbyte

	setb LOAD       ;uebernehmen der Daten ins Register
	clr LOAD        ;des MAX7219

	pop acc
	mov r1,a
	pop acc
	mov r0,a
	pop acc
	ret


;*************************************************
; Datentabellen

daten:
	db 1,3,7,15,31,63,127,255

daten_rev:
	db 254,252,248,240,224,192,128,0

END
